@@ -164,20 +164,24 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
 
             const bool hasLabel = !(*it).label().isEmpty();
             QMap<QString, Address::TypeFlag>::ConstIterator typeIt;
+            QStringList addreLineType;
+            QStringList labelLineType;
             for (typeIt = mAddressTypeMap.constBegin();
                     typeIt != mAddressTypeMap.constEnd(); ++typeIt) {
                 if (typeIt.value() & (*it).type()) {
-                    addParameter(adrLine, version, QStringLiteral("TYPE"), QStringList() << typeIt.key());
+                    addreLineType << typeIt.key();
                     if (hasLabel) {
-                        addParameter(labelLine, version, QStringLiteral("TYPE"), QStringList() << typeIt.key());
+                        labelLineType << typeIt.key();
                     }
                 }
             }
 
             if (!isEmpty) {
+                addParameter(adrLine, version, QStringLiteral("TYPE"), addreLineType);
                 card.addLine(adrLine);
             }
             if (hasLabel) {
+                addParameter(labelLine, version, QStringLiteral("TYPE"), labelLineType);
                 card.addLine(labelLine);
             }
         }
@@ -254,46 +258,30 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         const Email::List emailList = (*addrIt).emailList();
         Email::List::ConstIterator emailIt;
         Email::List::ConstIterator emailEnd(emailList.end());
-        bool pref = true;
         for (emailIt = emailList.begin(); emailIt != emailEnd; ++emailIt) {
-            bool needToAddPref = false;
             VCardLine line(QStringLiteral("EMAIL"), (*emailIt).mail());
-            if (pref == true && emailList.count() > 1) {
-                needToAddPref = true;
-                pref = false;
-            }
             QMapIterator<QString, QStringList> i((*emailIt).parameters());
-            bool foundType = false;
             while (i.hasNext()) {
                 i.next();
-                QStringList valueStringList = i.value();
-                if (i.key().toLower() == QLatin1String("type")) {
-                    if (!valueStringList.contains(QStringLiteral("PREF"))) {
-                        if (needToAddPref) {
-                            valueStringList.append(QStringLiteral("PREF"));
-                        } else {
-                            needToAddPref = false;
-                        }
-                    } else {
-                        if (!needToAddPref) {
-                            valueStringList.removeAll(QStringLiteral("PREF"));
-                        }
-                    }
-                    foundType = true;
-                }
-                if (!valueStringList.isEmpty()) {
-                    if (i.key().toLower() == QLatin1String("type")) {
-                        addParameter(line, version, i.key(), valueStringList);
-                    } else {
-                        line.addParameter(i.key(), valueStringList.join(QStringLiteral(",")));
-                    }
-                }
-            }
-            if (!foundType && needToAddPref) {
                 if (version == VCard::v2_1) {
-                    line.addParameter(QStringLiteral("PREF"), QString());
+                    if (i.key().toLower() == QLatin1String("type")) {
+                        QStringList valueStringList = i.value();
+                        bool hasPreferred = false;
+                        if (valueStringList.contains(QStringLiteral("PREF"))) {
+                            valueStringList.removeAll(QStringLiteral("PREF"));
+                            hasPreferred = true;
+                        }
+                        if (!valueStringList.isEmpty()) {
+                            addParameter(line, version, i.key(), valueStringList);
+                        }
+                        if (hasPreferred) {
+                            line.addParameter(QStringLiteral("PREF"), QString());
+                        }
+                    } else {
+                        line.addParameter(i.key(), i.value().join(QStringLiteral(",")));
+                    }
                 } else {
-                    line.addParameter(QStringLiteral("TYPE"), QStringLiteral("PREF"));
+                    line.addParameter(i.key(), i.value().join(QStringLiteral(",")));
                 }
             }
             card.addLine(line);
@@ -451,12 +439,16 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
 
             QMap<QString, PhoneNumber::TypeFlag>::ConstIterator typeIt;
             QMap<QString, PhoneNumber::TypeFlag>::ConstIterator typeEnd(mPhoneTypeMap.constEnd());
+            QStringList lst;
             for (typeIt = mPhoneTypeMap.constBegin(); typeIt != typeEnd; ++typeIt) {
                 if (typeIt.value() & (*phoneIt).type()) {
-                    addParameter(line, version, QStringLiteral("TYPE"), QStringList() << typeIt.key());
+                    lst << typeIt.key();
+
                 }
             }
-
+            if (!lst.isEmpty()) {
+                addParameter(line, version, QStringLiteral("TYPE"), lst);
+            }
             card.addLine(line);
         }
 
@@ -490,7 +482,6 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         card.addLine(VCardLine(QStringLiteral("UID"), (*addrIt).uid()));
 
         // URL
-        card.addLine(VCardLine(QStringLiteral("URL"), (*addrIt).url().url()));
         Q_FOREACH (const ResourceLocatorUrl &url, (*addrIt).extraUrlList()) {
             VCardLine line(QStringLiteral("URL"), url.url());
             QMapIterator<QString, QStringList> i(url.parameters());
@@ -1000,7 +991,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                         type |= mPhoneTypeMap[(*it).toUpper()];
                         foundType = true;
                     }
-                    phone.setType( foundType ? type : PhoneNumber::Undefined);
+                    phone.setType(foundType ? type : PhoneNumber::Undefined);
 
                     addr.insertPhoneNumber(phone);
                 }
@@ -1037,12 +1028,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     ResourceLocatorUrl resourceLocatorUrl;
                     resourceLocatorUrl.setUrl(url);
                     resourceLocatorUrl.setParameters((*lineIt).parameterMap());
-
-                    if (addr.url().url().isEmpty()) {
-                        addr.setUrl(resourceLocatorUrl);
-                    } else {
-                        addr.insertExtraUrl(resourceLocatorUrl);
-                    }
+                    addr.insertExtraUrl(resourceLocatorUrl);
                 }
                 // SOURCE
                 else if (identifier == QLatin1String("source")) {
@@ -1061,6 +1047,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     addr.insertRelationShip(related);
                 }
                 // X-
+                //TODO import X-GENDER
                 else if (identifier.startsWith(QLatin1String("x-"))) {
                     QString ident = (*lineIt).identifier();
                     //X-Evolution
